#include <msp/core/maputils.h>
#include <msp/strings/format.h>
#include "libr2c2/catalogue.h"
#include "libr2c2/driver.h"
#include "libr2c2/layout.h"
#include "libr2c2/vehicletype.h"
#include "engineer.h"
#include "newtraindialog.h"
#include "traindialog.h"

using namespace std;
using namespace Msp;
using namespace R2C2;

static string vehicle_type_name(const VehicleType *const &vehtype)
{
	return format("%s %s", vehtype->get_article_number(), vehtype->get_name());
}

NewTrainDialog::NewTrainDialog(Engineer &e):
	engineer(e),
	loco_types(vehicle_type_name)
{
	Loader::WidgetMap widgets;
	DataFile::load(*this, "data/newtraindialog.ui", widgets);

	drp_protocol = dynamic_cast<GLtk::Dropdown *>(get_item(widgets, "drp_protocol"));
	drp_protocol->set_data(protocols);
	ent_address = dynamic_cast<GLtk::Entry *>(get_item(widgets, "ent_address"));
	ent_name = dynamic_cast<GLtk::Entry *>(get_item(widgets, "ent_name"));
	drp_loco_type = dynamic_cast<GLtk::Dropdown *>(get_item(widgets, "drp_loco_type"));
	drp_loco_type->set_data(loco_types);

	const Driver &driver = engineer.get_layout().get_driver();
	for(unsigned i=0;; ++i)
	{
		if(const char *proto = driver.enumerate_protocols(i))
			protocols.append(proto);
		else
			break;
	}

	drp_protocol->set_selected_index(0);

	list<VehicleType *> veh_types = engineer.get_layout().get_catalogue().get_list<VehicleType>();
	for(list<VehicleType *>::iterator i=veh_types.begin(); i!=veh_types.end(); ++i)
		if((*i)->is_locomotive())
			loco_types.append(*i);
}

void NewTrainDialog::prefill(const Driver::DetectedLocomotive &loco)
{
	drp_protocol->set_selected_index(protocols.find(loco.protocol));
	ent_address->set_text(lexical_cast<string>(loco.address));
	ent_name->set_text(loco.name);
}

void NewTrainDialog::on_response(int code)
{
	if(code)
	{
		int index = drp_protocol->get_selected_index();
		if(index<0)
			return;
		string protocol = protocols.get(index);

		unsigned address = lexical_cast<unsigned>(ent_address->get_text());

		index = drp_loco_type->get_selected_index();
		if(index<0)
			return;
		const VehicleType &type = *loco_types.get(index);

		Train *train = new Train(engineer.get_layout(), type, address, protocol);
		train->set_name(ent_name->get_text());

		TrainDialog *dlg = new TrainDialog(engineer, *train);
		find_ancestor<GLtk::Root>()->add(*dlg);
		dlg->autosize();
	}
}
