#ifndef LIBR2C2_VEHICLEPLACEMENT_H_
#define LIBR2C2_VEHICLEPLACEMENT_H_

#include <vector>
#include "trackoffsetiter.h"
#include "vehicletype.h"

namespace R2C2 {

class VehiclePlacement
{
public:
	enum Anchor
	{
		CENTER,
		FRONT_AXLE,
		FRONT_BUFFER,
		BACK_AXLE,
		BACK_BUFFER
	};

private:
	struct Axle
	{
		const VehicleType::Axle *type;
		TrackOffsetIter position;

		Axle(const VehicleType::Axle &);
	};

	const VehicleType &type;
	std::vector<Axle> axles;
	float front_back_span;
	float front_back_ratio;

public:
	VehiclePlacement(const VehicleType &);

	void place(const TrackOffsetIter &, Anchor = CENTER);
private:
	float get_anchor_position(Anchor) const;
public:
	void place_after(const VehiclePlacement &);
	void place_before(const VehiclePlacement &);
	void unplace();
	bool is_placed() const;

	OrientedPoint get_point() const;
	const TrackOffsetIter &get_axle_position(unsigned) const;
	TrackOffsetIter get_position(Anchor) const;
	OrientedPoint get_axle_point(unsigned) const;
	OrientedPoint get_bogie_point(unsigned) const;
private:
	OrientedPoint get_front_point() const;
	OrientedPoint get_back_point() const;
	OrientedPoint get_bogie_point_from_axle(unsigned) const;
	static OrientedPoint create_point(const Vector &, const Vector &, float);

public:
	void advance(float, Anchor = CENTER);
private:
	void fix_positions(float);
	void fix_bogie_position(unsigned);
	static float compute_adjustment(const Vector &, const Vector &, float, float &);
};

} // namespace R2C2

#endif
