#ifndef LIBR2C2_ZONE_H_
#define LIBR2C2_ZONE_H_

#include <set>
#include <string>
#include <msp/datafile/objectloader.h>
#include "trackchain.h"

namespace R2C2 {

class Layout;
class Track;

class Zone: public TrackChain
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Zone>
	{
	public:
		Loader(Zone &);
	private:
		virtual void finish();
		void block(unsigned);
		void direction_hint(unsigned, Direction);
		void preferred_direction(Direction);
	};

private:
	std::string group;
	std::string qualifier;
	unsigned number;
	int up_end;
	Direction preferred_dir;

public:
	Zone(Layout &);
	~Zone();

	virtual void set_name(const std::string &);
	void set_name(const std::string &, const std::string &, unsigned);
private:
	void update_name();
public:
	const std::string &get_group() const { return group; }
	const std::string &get_qualifier() const { return qualifier; }
	unsigned get_number() const { return number; }

private:
	virtual void on_track_added(Track &);

public:
	void set_direction_towards(Track &, Direction);
	bool has_direction() const { return up_end>=0; }
	void set_preferred_direction(Direction);
	Direction get_preferred_direction() const { return preferred_dir; }
	void clear_direction();
	virtual TrackIter iter_for(Track &, Direction) const;
private:
	TrackIter next_iter(const TrackIter &) const;
public:
	using TrackChain::get_end;
	virtual TrackIter get_end(Direction) const;

	void save(std::list<Msp::DataFile::Statement> &) const;
	virtual Msp::DataFile::Statement save_reference() const;
};

} // namespace R2C2

#endif
