#ifndef R2C2_NET_TRAIN_H_
#define R2C2_NET_TRAIN_H_

#include <sigc++/signal.h>
#include "libr2c2/vehicletype.h"
#include "packets.h"

namespace R2C2 {

class Client;

class NetTrain
{
public:
	sigc::signal<void, const std::string &> signal_name_changed;
	sigc::signal<void, const std::string &, float> signal_control_changed;
	sigc::signal<void, unsigned, bool> signal_function_changed;
	sigc::signal<void, const std::string &> signal_route_changed;
	sigc::signal<void, const std::string &> signal_status_changed;

private:
	Client &client;
	const VehicleType &loco_type;
	unsigned address;
	std::string name;
	std::map<std::string, float> controls;
	unsigned functions;
	std::string route;
	std::string status;

public:
	NetTrain(Client &, const TrainInfoPacket &);

	const VehicleType &get_loco_type() const { return loco_type; }
	unsigned get_address() const { return address; }
	const std::string &get_name() const { return name; }
	void set_control(const std::string &, float);
	float get_control(const std::string &) const;
	void set_function(unsigned, bool);
	bool get_function(unsigned i) const { return (functions>>i)&1; }
	void set_route(const std::string &);
	const std::string &get_route() const { return route; }

	void process_packet(const TrainControlPacket &);
	void process_packet(const TrainFunctionPacket &);
	void process_packet(const TrainRoutePacket &);
	void process_packet(const TrainStatusPacket &);
};

} // namespace R2C2

#endif
